"""Reward models for evaluating responses."""

from databricks.kie.research_common.reward_model_utils import DEFAULT_CONTEXT_REWARD_MODEL_ID, generate_reward_rating


class RewardModel():
    """Model that assigns a reward to a response."""

    def __call__(self, prompt: str, response: str) -> float:
        raise NotImplementedError()


class ContextRewardModel(RewardModel):
    """Reward model that gets reward score by prompting GPT."""

    def __init__(self, model_id: str = DEFAULT_CONTEXT_REWARD_MODEL_ID):
        self.model_id = model_id

    def __call__(self, prompt: str, response: str) -> float:
        return float(generate_reward_rating(prompt, response, self.model_id))



"""Module to encapsulate LLM invocation logic as custom MLFLow PythonModel."""

from typing import Dict, Optional

import mlflow
from mlflow.models import ModelSignature
from mlflow.pyfunc import PythonModelContext

from databricks.kie.t2t_models.base_t2t_model import BaseT2TModel


class CustomModel(mlflow.pyfunc.PythonModel):
    """
    A custom MLflow Python model that processes inputs based on provided instructions.

    Args:
        model (BaseT2TModel): The model to use for generating predictions.
        signature (ModelSignature): The signature of the model.
        model_descriptor (str): The descriptor of the model.
    """

    def __init__(self, model: BaseT2TModel, signature: ModelSignature, model_descriptor: str = None):
        self.model = model
        self.model_descriptor = model_descriptor
        self.signature = signature

    def predict(self, context: PythonModelContext, model_input: str, params: Optional[Dict] = None):
        del context, params
        return self.model(model_input)


class ModelWrapper():

    def __init__(self,
                 custom_model: CustomModel,
                 model_run_id: str,
                 run_name: str,
                 registered_model_name: Optional[str] = None):
        self.custom_model = custom_model
        self.run_id = model_run_id
        self.run_name = run_name
        self.registered_model_name = registered_model_name

    def get_model_path(self):
        return f"runs:/{self.run_id}/model"



"""Base class interface for T2T models."""

from abc import ABC, abstractmethod
from typing import Optional

from databricks.kie.t2t_schema import T2TSystemParams


class BaseT2TModel(ABC):
    """Abstract base class for text-to-text models."""

    @abstractmethod
    def __call__(self, model_input: str) -> str:
        """Generate prediction for the given input.
        
        Args:
            model_input: Input text to generate prediction for

        Returns:
            Generated prediction text
        """
        raise NotImplementedError

    @staticmethod
    @abstractmethod
    def create_from_system_param(system_param: T2TSystemParams, **kwargs) -> Optional['BaseT2TModel']:
        """Create a model instance from a system parameter.
        
        Args:
            system_param: System parameter containing instruction and example data

        Returns:
            Instantiated model
        """
        raise NotImplementedError



"""Convenience module to import all T2T models."""
from databricks.kie.t2t_models.best_of_n_model import BestOfNModel  # pylint: disable=unused-import
from databricks.kie.t2t_models.t2t_baseline_model import T2TBaselineModel  # pylint: disable=unused-import
from databricks.kie.t2t_models.t2t_prompt_tuning_model import T2TPromptTuningModel  # pylint: disable=unused-import
